package App::Adenosine::Role::WrapsCurlCommand;
{
  $App::Adenosine::Role::WrapsCurlCommand::VERSION = '2.000001';
}

use Moo::Role;

requires 'wrap';

1;

__END__

=pod

=head1 NAME

App::Adenosine::Role::WrapsCurlCommand

=head1 VERSION

version 2.000001

=head1 AUTHOR

Arthur Axel "fREW" Schmidt <frioux+cpan@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Arthur Axel "fREW" Schmidt.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
